@@ -8,9 +8,6 @@
 #
 # The full license is in the file COPYING, distributed with this software.
 #-----------------------------------------------------------------------------
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
 import importlib
 import inspect
 import os
@@ -800,7 +797,8 @@ class FitTab(NXTab):
                                       params=self.parameters,
                                       weights=weights,
                                       x=self.axis,
-                                      method=self.method)
+                                      method=self.method,
+                                      nan_policy='omit')
         except Exception as error:
             report_error("Fitting Data", error)
         if self.fit:
