@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 #-----------------------------------------------------------------------------
-# Copyright (c) 2013, NeXpy Development Team.
+# Copyright (c) 2013-2020, NeXpy Development Team.
 #
 # Distributed under the terms of the Modified BSD License.
 #
@@ -12,9 +12,6 @@
 """
 Base class for import dialogs
 """
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
 import os
 
 from nexusformat.nexus import *
