@@ -1,16 +1,22 @@
+#!/usr/bin/env python
 # -*- coding: utf-8 -*-
+
+#-----------------------------------------------------------------------------
+# Copyright (c) 2018-2020, NeXpy Development Team.
+#
+# Distributed under the terms of the Modified BSD License.
+#
+# The full license is in the file COPYING, distributed with this software.
+#-----------------------------------------------------------------------------
+
 """
 This module contains a set of customized widgets both for dialogs and plot objects. 
 """
-from __future__ import absolute_import, division, unicode_literals
-
 import warnings
 
 import matplotlib as mpl
 import math
 import numpy as np
-import re
-import six
 from matplotlib import colors
 from matplotlib import cbook
 from matplotlib.patches import Circle, Ellipse, Polygon, Rectangle
@@ -85,7 +91,7 @@ class NXStack(QtWidgets.QWidget):
 class NXSortModel(QtCore.QSortFilterProxyModel):
 
     def __init__(self, parent=None):
-        super(NXSortModel, self).__init__(parent)
+        super(NXSortModel, self).__init__(parent=parent)
 
     def lessThan(self, left, right):
         left_text = self.sourceModel().itemFromIndex(left).text()
@@ -112,7 +118,7 @@ class NXScrollArea(QtWidgets.QScrollArea):
         if not horizontal:
             self.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
         self.setWidgetResizable(True)
-        self.setSizePolicy(QtWidgets.QSizePolicy.Minimum,
+        self.setSizePolicy(QtWidgets.QSizePolicy.Expanding,
                            QtWidgets.QSizePolicy.Expanding)
 
 
@@ -199,7 +205,7 @@ class NXLineEdit(QtWidgets.QLineEdit):
         super(NXLineEdit, self).__init__(parent=parent)
         if slot:
             self.editingFinished.connect(slot)
-        if text:
+        if text is not None:
             self.setText(text)
         if width:
             self.setFixedWidth(width)
@@ -233,7 +239,7 @@ class NXTextBox(NXLineEdit):
         float
             Value of text box converted to a floating point number
         """
-        return float(six.text_type(self.text()))
+        return float(str(self.text()))
 
     def setValue(self, value):
         """Set the value of the text box string formatted as a float.
@@ -243,7 +249,34 @@ class NXTextBox(NXLineEdit):
         value : str or int or float
             Text box value to be formatted as a float        
         """
-        self.setText(six.text_type(float('%.4g' % value)))
+        self.setText(str(float('%.4g' % value)))
+
+
+class NXPlainTextEdit(QtWidgets.QPlainTextEdit):
+    """An editable text window."""
+
+    def __init__(self, text=None, wrap=True, parent=None):
+        super(NXPlainTextEdit, self).__init__(parent=parent)
+        self.setFont(QtGui.QFont('Courier'))
+        if not wrap:
+            self.setLineWrapMode(QtWidgets.QPlainTextEdit.NoWrap)
+        if text:
+            self.setPlainText(text)
+        self.setFocusPolicy(QtCore.Qt.StrongFocus)
+
+    def __repr__(self):
+        return 'NXPlainTextEdit()'
+
+    def setPlainText(self, text):
+        """Function to set the text in the window.
+
+        Parameters
+        ----------
+        text : str
+            Text to replace the text box contents.        
+        """
+        super(NXPlainTextEdit, self).setPlainText(str(text))
+        self.repaint()
 
 
 class NXMessageBox(QtWidgets.QMessageBox):
@@ -282,12 +315,20 @@ class NXComboBox(QtWidgets.QComboBox):
         self.setFocusPolicy(QtCore.Qt.StrongFocus)
         self.setMinimumWidth(100)
         if items:
-            self.addItems(items)
+            self.addItems([str(item) for item in items])
             if default:
                 self.setCurrentIndex(self.findText(default))
         if slot:
             self.activated.connect(slot)
 
+    def __iter__(self):
+        """Implement key iteration."""
+        return self.items().__iter__()
+            
+    def __next__(self):
+        """Implements key iteration."""
+        return self.items().__next__()
+            
     def __contains__(self, item):
         """True if the item is one of the options."""
         return item in self.items()
@@ -313,6 +354,26 @@ class NXComboBox(QtWidgets.QComboBox):
         else:
             self.parent().keyPressEvent(event)
 
+    def findText(self, value, **kwargs):
+        """Function to return the index of a text value.
+        
+        This is needed since h5py now returns byte strings, which will trigger
+        ValueErrors unless they are converted to unicode strings.
+        
+        Parameters
+        ----------
+        value :
+            Searched value.  
+
+        Returns
+        -------
+        int
+            Index of the searched value. 
+        """
+        if isinstance(value, bytes):
+            value = value.decode('utf-8')
+        return super(NXComboBox, self).findText(str(value), **kwargs)
+
     def add(self, *items):
         """Add items to the list of options.
 
@@ -372,6 +433,7 @@ class NXComboBox(QtWidgets.QComboBox):
             The option to be selected in the dropdown menu.
         """
         self.setCurrentIndex(self.findText(str(item)))
+        self.repaint()
 
     @property
     def selected(self):
@@ -441,7 +503,7 @@ class NXPushButton(QtWidgets.QPushButton):
         parent : QObject, optional
             Parent of button.
         """
-        super(NXPushButton, self).__init__(label, parent)
+        super(NXPushButton, self).__init__(label, parent=parent)
         self.setFocusPolicy(QtCore.Qt.StrongFocus)
         self.setDefault(False)
         self.setAutoDefault(False)
@@ -471,7 +533,7 @@ class NXColorButton(QtWidgets.QPushButton):
     colorChanged = QtCore.Signal(QtGui.QColor)
 
     def __init__(self, parent=None):
-        super(NXColorButton, self).__init__(parent)
+        super(NXColorButton, self).__init__(parent=parent)
         self.setFixedWidth(18)
         self.setStyleSheet("width:18px; height:18px; "
                            "margin: 0px; border: 0px; padding: 0px;"
@@ -531,7 +593,7 @@ class NXColorBox(QtWidgets.QWidget):
         parent : QObject, optional
             Parent of the color box.
         """
-        super(NXColorBox, self).__init__(parent)
+        super(NXColorBox, self).__init__(parent=parent)
         self.color_text = color
         color = self.qcolor(self.color_text)
         self.layout = QtWidgets.QHBoxLayout()
@@ -544,7 +606,7 @@ class NXColorBox(QtWidgets.QWidget):
                                   parent=parent, slot=self.update_color, 
                                   width=width, align='right')
         self.layout.addWidget(self.textbox)
-        self.button = NXColorButton(parent)
+        self.button = NXColorButton(parent=parent)
         self.button.color = color
         self.button.colorChanged.connect(self.update_text)
         self.layout.addWidget(self.button)
@@ -676,7 +738,7 @@ class NXSpinBox(QtWidgets.QSpinBox):
         self.repaint()
 
     def valueFromText(self, text):
-        return self.indexFromValue(float(six.text_type(text)))
+        return self.indexFromValue(float(str(text)))
 
     def textFromValue(self, value):
         try:
@@ -784,14 +846,9 @@ class NXDoubleSpinBox(QtWidgets.QDoubleSpinBox):
     def setSingleStep(self, value):
         value = abs(value)
         if value == 0:
-            self.setDecimals(2)
             stepsize = 0.01
         else:
             digits = math.floor(math.log10(value))
-            if digits < 0:
-                self.setDecimals(-digits)
-            else:
-                self.setDecimals(2)
             multiplier = 10**digits
             stepsize = find_nearest(self.steps, value/multiplier) * multiplier
         super(NXDoubleSpinBox, self).setSingleStep(stepsize)
@@ -818,6 +875,14 @@ class NXDoubleSpinBox(QtWidgets.QDoubleSpinBox):
             return format_float(value, width=8)
 
     def setValue(self, value):
+        if value == 0:
+            self.setDecimals(2)
+        else:
+            digits = math.floor(math.log10(abs(value)))
+            if digits < 0:
+                self.setDecimals(-digits)
+            else:
+                self.setDecimals(2)
         if value > self.maximum():
             self.setMaximum(value)
         elif value < self.minimum():
